import 'dart:async';
import 'dart:io';

import 'package:glob/glob.dart';
import 'package:io/ansi.dart' as ansi;
import 'package:path/path.dart' as p;

import 'utils.dart';

Future<void> runBuildRunner(
  String pkgDirectory,
  Map<String, String> targets,
  String config,
  bool release,
) async {
  final targetsValue =
      targets.entries.map((e) => '${e.key}:${e.value}').join(',');

  final args = [
    'run',
    'build_runner',
    'build',
    release ? '--release' : '--no-release',
  ];

  if (config != null) {
    args.addAll(['--config', config]);
  }

  args.addAll([
    '--output',
    targetsValue,
  ]);

  print(ansi.styleBold.wrap('''
Command:     ${['pub'].followedBy(args).join(' ')}
'''));

  await runProcess(pubPath, args, workingDirectory: pkgDirectory);

  var deleteCount = 0;

  for (var buildDir in targets.values) {
    for (var file in Directory(buildDir)
        .listSync(recursive: true, followLinks: false)
        .whereType<File>()) {
      final relativePath = p.relative(file.path, from: buildDir);

      if (_badFileGlob.matches(relativePath)) {
        if (deleteCount == 0) {
          stdout.write('Deleting extra files from output directory');
        }
        stdout.write('.');
        file.deleteSync();
        deleteCount++;
      }
    }
  }
  if (deleteCount > 0) {
    // Ensure we add a new line is added after printing `.` for deleted files
    print('');
  }
}

const _globItems = {
  '**.dart',
  '**.dart.js.deps',
  '**.dart.js.tar.gz',
  '**.md',
  '**.module',
  '**.ng_placeholder', // Generated by pkg:angular
  '**.yaml',
  '.build.manifest',
  '.packages',
  'packages/\$sdk/**',
  'packages/analyzer/**',
  'packages/build_runner/**',
  'packages/build_web_compilers/**',
  'packages/node_preamble/**',
  'packages/package_resolver/**',
  'packages/test/**',
};

final _badFileGlob = Glob('{${_globItems.join(',')}}');
